indexing
   description: "Main drawing window and access to the image"
   "generation and manipulation."
   status: "Free according to the GPL license (see COPYING)"
   date: "$Date$"
   revision: "$Revision$"
   author: "Mark Arrasmith <arrasmith@math.twsu.edu> and Miron Brezuleanu"
   

class EFRACTAL_CANVAS
   
inherit
   GTK_DRAWING_AREA
      redefine
	 make,
	 expose_handler,
	 button_1_press_handler,
	 button_1_release_handler,
	 button_2_release_handler,
	 button_3_release_handler,
	 motion_notify_handler,
	 key_release_handler
      end
   EXG_CONSTANTS
   GTK_TIMEOUT_IDLE_COMMAND
   EXG_TIME
   STARTUP
   EFRACTAL_SELECTION
   EXG_STORABLE
   
creation
   make
   
feature
   
   make is
      do
	 Precursor
	 activate_event_handling
	 set_size(config.width, config.height)
	 !!idle.make(Current, 0)
	 da := False
	 set_flags (gtk_can_focus)
	 selection := Void
	 step := 0
	 dragging := False
	 create_black_image
	 initialize_efractal
      end
   
   create_black_image is
      do
	 !!image.make_with_defaults(config.width,config.height)
      end
   
   initialize_efractal is
      do
	 gradient := colors.gradient
	 x := 0
	 y := 0
	 line_drawn := False
	 image.destroy
	 create_black_image
      end
   
   generate is
      do
	 if idle /= Void then
	    idle.remove	 
	 end
	 !!idle.make(Current, 0)
	 initialize_efractal
      end
    
   draw_background is
      local
	 gc: GDK_GC
      do
	 !! gc.make (Current)
	    gc.set_foreground(exg_white)
	    draw_image(gc, image, 0, 0, 0, 0, image.width, image.height)
	 gc.destroy
      end
   
   zoom is
	 -- uses the selection to whatever purpose ...
      require
	 selection /= Void
      do
	-- draw fractal
	-- screen to complex transforms
	config.inc_zoom
	config.set_x1(screen_to_complex_x(selection.points.item(1).x,selection.points.item(1).y))
	config.set_x2(screen_to_complex_x(selection.points.item(2).x,selection.points.item(2).y))
	config.set_x3(screen_to_complex_x(selection.points.item(3).x,selection.points.item(3).y))
	config.set_x4(screen_to_complex_x(selection.points.item(4).x,selection.points.item(4).y))
	config.set_y1(screen_to_complex_y(selection.points.item(1).x,selection.points.item(1).y))
	config.set_y2(screen_to_complex_y(selection.points.item(2).x,selection.points.item(2).y))
	config.set_y3(screen_to_complex_y(selection.points.item(3).x,selection.points.item(3).y))
	config.set_y4(screen_to_complex_y(selection.points.item(4).x,selection.points.item(4).y))
	-- create transform constants
	config.calculate_transform
	-- call fractal
	clear_zoombox
	initialize_efractal
	generate
      end
   
   reset is
	 -- prepares the widget for a new selection
      do
	 selection := Void
	 step := 0
      end
   
   expose_handler (area: GDK_RECTANGLE) is
	 -- redraws the canvas
      local
	 gc: GDK_GC
	 r: GDK_RECTANGLE
      do
	 draw_background
	 da := True
	 !! gc.make (Current)
	 inspect step
	 when 0 then
	 when 1 then
	    gc.set_function (gdk_xor)
	    gc.set_foreground (exg_white) 
	    -- `exg_white' is essential for the xor stuff to work!!
	    r := rectangle_from_rubberband
		gc.set_line_width(2)
	    draw_rectangle (gc, False,
			    r.x, r.y, r.width, r.height)
	 when 2 then
	    selection.draw (Current)
	 end
	 gc.destroy
      end
    
   new_type is
      local
	  ntd: NEW_TYPES_DIALOG
	  do
	  !!ntd.make
	  ntd.init_dialog
	  ntd.run
	  if equal(ntd.answer, "ok") then
		 config.reinitialize_window
		 config.set_do_julia(False)
		 clear_zoombox
		 initialize_efractal
		 generate
	  end
	  end

   new_params is
      local
	  npd: NEW_PARAMS_DIALOG
	  do
	  !!npd.make
	  npd.init_dialog
	  npd.run
	  if equal(npd.answer, "ok") then
		 clear_zoombox
		 initialize_efractal
		 generate
	  end
	  end
	  
   clear_zoombox is
      do
	     reset
		 draw_background
	  end
   
   new_zoombox is
      local
	  nzd: NEW_ZOOMBOX_DIALOG
	  do
	  !!nzd.make
	  nzd.init_dialog
	  nzd.run
	  if equal(nzd.answer, "ok") then
	     config.calculate_transform
		 clear_zoombox
		 initialize_efractal
		 generate
	  end
	  end
	  
   undo_zoom is
      do
	  if config.zoom.count1 > 1 then
	     config.dec_zoom
	     config.calculate_transform
	     clear_zoombox
		 initialize_efractal
	     generate
	  end
	  end
   
   new_colors is
      local
	  ncd: NEW_COLORS_DIALOG
	  do
	  !!ncd.make
	  ncd.init_dialog
	  ncd.run
	  if equal(ncd.answer, "ok") then
		 clear_zoombox
		 initialize_efractal
		 generate
	  end
	  end
   
   new_size is
      local
	 ncd: NEW_SIZE_DIALOG
      do
	 !!ncd.make
	 ncd.init_dialog
	 ncd.run
	 if equal(ncd.answer, "ok") then
	    set_size(config.width, config.height)
		config.calculate_transform
	    clear_zoombox
		initialize_efractal
	    generate
	 end
      end
   
   save_image is
      local
	 efsm: EXG_FILE_SELECTION_MODAL
	 gc: GDK_GC
	 pixmap: GDK_PIXMAP
	 ii: IMLIB_IMAGE
	 file: STRING
	 idx: INTEGER
	 split_at: CHARACTER
      do
	 !!efsm.make
	 efsm.set_title("Save Image")
	 efsm.set_filename(config.image_path)
	 efsm.run
	 if equal(efsm.answer, "ok") then
	    !!gc.make(Current)
	    !!pixmap.make(image.width, image.height)
	    pixmap.draw_image(gc, image, 0, 0, 0, 0,
			      image.width, image.height)
	    !!ii.make(pixmap, 0, 0, image.width, image.height)
	    gc.destroy
	    pixmap.destroy
		if efsm.filename.has_suffix(".png") or efsm.filename.has_suffix(".ppm") or efsm.filename.has_suffix(".jpg") then
	        ii.save(efsm.filename)
	        ii.destroy
		else
		    efsm.filename.append(".png")
			ii.save(efsm.filename)
			ii.destroy
		end
		file := "temp"
		file.copy(efsm.filename)
		if file.has('\') then
		  split_at := '\'
		else
		  split_at := '/'
		end
		file.reverse
		idx := file.index_of(split_at)
		file.head(idx - 1)
		file.reverse
		efsm.filename.remove_suffix(file)
		config.set_image_path(efsm.filename)
		
	 end
      end
	  
    save_to_file(file: STD_FILE_WRITE) is
	 -- save the fractal parameters
      local
	 i: INTEGER
      do
	  file.put_string("type " + config.type + "%N")
	  file.put_string("do_julia " + config.do_julia.to_string + "%N")
	  file.put_string("julia_seed " + config.julia_x.to_string_format(18) + " " + config.julia_y.to_string_format(18) + "%N")
	  file.put_string("lyapunov " + config.lyapunov_pattern + "%N")
	  file.put_string("size " + config.width.to_string + " " + config.height.to_string + "%N")
	  file.put_string("dump " + config.dump.to_string + "%N")
	  file.put_string("iterations " + config.total.to_string + "%N")
	  file.put_string("color_outside " + config.color_outside + "%N")
	  file.put_string("color_inside " + config.color_inside + "%N")
	  file.put_string("num_colors " + config.colors.to_string + "%N")
	  file.put_string("point1 " + config.x1.to_string_format(18) + " " + config.y1.to_string_format(18) + "%N")
	  file.put_string("point2 " + config.x2.to_string_format(18) + " " + config.y2.to_string_format(18) + "%N")
	  file.put_string("point3 " + config.x3.to_string_format(18) + " " + config.y3.to_string_format(18) + "%N")
	  file.put_string("point4 " + config.x4.to_string_format(18) + " " + config.y4.to_string_format(18) + "%N")
	  file.put_string("color_map%N")
	  from
	     i := colors.palette.lower
	  until
	     i > colors.palette.upper
	  loop
	     file.put_string(colors.palette.item(i).red.to_string + " ")
	     file.put_string(colors.palette.item(i).green.to_string + " ")
	     file.put_string(colors.palette.item(i).blue.to_string + "%N")
		 i := i + 1
	  end
	  file.put_string("end%N")
      end      
      
    load_from_file(file: STD_FILE_READ) is
	 -- will load the parameters from the given file
      local
	 stage, i : INTEGER	 
	 components: ARRAY[STRING]
	 color: GDK_COLOR
      do
	 from
	    stage := 0
	 until
	    file.end_of_input or stage = 1 
	 loop
	    file.read_line
		inspect stage
		when 0 then
		if equal(file.last_string, "end") then
		  stage := 1 
		else 
		  components := file.last_string.split
		  if equal(components.item(1), "type") then
		    config.set_type(components.item(2))
		  elseif equal(components.item(1), "do_julia") then
		    config.set_do_julia(components.item(2).same_as("true"))
		  elseif equal(components.item(1), "julia_seed") then
		    config.set_julia_x(components.item(2).to_double)
		    config.set_julia_y(components.item(3).to_double)
		  elseif equal(components.item(1), "lyapunov") then
		    config.set_lyapunov_pattern(components.item(2))
		  elseif equal(components.item(1), "size") then
		    config.set_width(components.item(2).to_integer)
		    config.set_height(components.item(3).to_integer)
		  elseif equal(components.item(1), "dump") then
		    config.set_dump(components.item(2).to_integer)
		  elseif equal(components.item(1), "iterations") then
		    config.set_total(components.item(2).to_integer)
		  elseif equal(components.item(1), "color_outside") then
		    config.set_color_outside(components.item(2))
		  elseif equal(components.item(1), "color_inside") then
		    config.set_color_inside(components.item(2))
		  elseif equal(components.item(1), "num_colors") then
		    config.set_colors(components.item(2).to_integer)
		  elseif equal(components.item(1), "point1") then
		    config.load_zoom
		    config.set_x1(components.item(2).to_double)
		    config.set_y1(components.item(3).to_double)
		  elseif equal(components.item(1), "point2") then
		    config.set_x2(components.item(2).to_double)
		    config.set_y2(components.item(3).to_double)
		  elseif equal(components.item(1), "point3") then
		    config.set_x3(components.item(2).to_double)
		    config.set_y3(components.item(3).to_double)
		  elseif equal(components.item(1), "point4") then
		    config.set_x4(components.item(2).to_double)
		    config.set_y4(components.item(3).to_double)
		  elseif equal(components.item(1), "color_map") then
		    from
		      i := 1
		    until
		      i > 12
		    loop
		      file.read_line
			  components := file.last_string.split
			  !!color.make(components.item(1).to_integer,
			          components.item(2).to_integer,
			  		  components.item(3).to_integer)
			  colors.palette.put(color,i)
			  i := i + 1
		    end
		  end
		end
		end
	  end -- end loop
	   set_size(config.width, config.height)
	   config.calculate_transform
	   clear_zoombox
	   initialize_efractal
	   generate
	end
  
   signature: STRING is
      do
	 Result := "eFractal parameter file"
      end

   julia(fractal_name: STRING): BOOLEAN is
      local
	i: INTEGER
      do
	from
	   i := 1
	   Result := False
	until
	   i > julia_types.count
	loop
	   if fractal_name.is_equal(julia_types.item(i)) then
	     Result := True
	   end
	   i := i + 1
	end
      end -- end julia

   screen_to_complex_x(sx,sy: DOUBLE): DOUBLE is
      local
      do
	Result := config.a*sx + config.b*sy + config.g*sx*sy + config.c
      end

   screen_to_complex_y(sx,sy: DOUBLE): DOUBLE is
      local
      do
	Result := config.d*sx + config.e*sy + config.h*sx*sy + config.f
      end 
  
   execute(q: ANY): BOOLEAN is      
      local
	 i, pixel: INTEGER
      do
	 from
	    i := 1
	 until
	    i > 5
	       or (x = 0 and y = image.height)
	 loop
        a1 := screen_to_complex_x(x,y)
	    b1 := screen_to_complex_y(x,y)
	    pixel := generate_fractal(a1,b1)
	    pixel := (pixel \\ gradient.count)+1
	    image.put_pixel(x, y, gradient.item(pixel))
	    x := x + 1
	    if x = image.width then
	       x := 0
	       y := y + 1
		   line_drawn := False
	    end
	    i := i + 1
	 end
	 if x = 0 and y = image.height then
	    expose_handler(Void)
	    idle := Void
	    Result := False
	 else
		if y \\ 30 = 0 and not line_drawn then
		   line_drawn := True
	       expose_handler(Void)
	    end
	    Result := True
	 end
      end

   button_1_press_handler (eb: GDK_EVENT_BUTTON) is
	 -- handles the creation & modification of the selection
      do
	 expect_key_press := False
	 inspect step
	 when 0 then
	    dragging := True
	    !! p1.make (eb.int_x, eb.int_y)
	    !! p2.make (eb.int_x, eb.int_y)
	    step := 1
	    expose_handler (Void)
	 when 1 then
	 when 2 then
	    !! p1.make (eb.int_x, eb.int_y)
	    if not selection.matches_point (p1) then 
	       -- the old `selection restart' code
	       step := 0
	       button_1_press_handler (eb) -- nasty kludge
	       
	       selection := Void
	       -- _plus_ this line
	    else
	       dragging := True
	    end
	 end
      end
   
   button_1_release_handler (eb: GDK_EVENT_BUTTON) is
	 -- handles the creation & modification of the selection
      do
	 expect_key_press := False
	 inspect step
	 when 0 then
	 when 1 then
	    dragging := False
	    step := 2
	    !! selection.make (rectangle_from_rubberband)
	    expose_handler (Void)
	 when 2 then
	    if dragging then
	       dragging := False
	       expose_handler (Void)
	    end
	 end
      end
   
   button_2_release_handler (eb: GDK_EVENT_BUTTON) is
	 -- displays the current selection (if there is one)
      local
	 aux: GDK_POINT
      do
	 expect_key_press := False
	 inspect step
	 when 0 then
	 when 1 then
	 when 2 then
	    !! aux.make (eb.int_x, eb.int_y)
	    if selection.matches_point (aux) then
	       expect_key_press := True
	    end
	 end
      end
   
   button_3_release_handler (eb: GDK_EVENT_BUTTON) is
	 -- displays the current selection (if there is one)
      do
	 expect_key_press := False
	 inspect step
	 when 0 then
	  if julia(config.type) = True then
	    if config.do_julia = False then
	      jx := eb.x.rounded
	      jy := eb.y.rounded
	      config.set_julia_x(screen_to_complex_x(jx,jy))
	      config.set_julia_y(screen_to_complex_y(jx,jy))
	      config.set_do_julia(True)
	      --config.reinitialize_window
		  config.start_julia
	      clear_zoombox
	      initialize_efractal
	      generate
	    else
	      config.set_do_julia(False)
	      --config.reinitialize_window
		  config.end_julia
	      clear_zoombox
		  initialize_efractal
	      generate
	    end
	  end
	 when 1 then
	 when 2 then
	    zoom
	    reset
	 end
      end
   
   motion_notify_handler (em: GDK_EVENT_MOTION) is
	 -- handles the creation & modification of the selection
      local
	 aux: GDK_POINT
      do
	 inspect step
	 when 0 then
	 when 1 then
	    if dragging then
	       !! p2.make (em.int_x, em.int_y)
	       expose_handler (Void)
	    end
	 when 2 then
	    if dragging then
	       !! aux.make (em.int_x, em.int_y)
	       selection.points.put (aux, selection.matched_point_index)
	       expose_handler (Void)
	    end
	 end
      end
   
   key_release_handler (ek: GDK_EVENT_KEY) is
	 -- handles the creation & modification of the selection
      local
	 aux: CHARACTER
      do
	 inspect step
	 when 0 then
	 when 1 then
	 when 2 then
	    if expect_key_press 
	       and ek.keyval >= ('1').to_integer
	       and ek.keyval <= ('6').to_integer then
	       aux := ek.keyval.to_character
	       inspect aux
	       when '1'..'4' then
		  selection.set_index (ek.keyval - ('0').to_integer,
				       selection.matched_point)
	       when '5' then
		  selection.rotate_left
	       when '6' then
		  selection.rotate_right
	       end
	       expose_handler (Void)
	    end
	 end
      end
   
feature -- queries & attributes
   
   rectangle_from_rubberband: GDK_RECTANGLE is
	 -- will use p1, p2 to give a rectangle as we usually expect 
	 -- one to be (location of top corner, width, height)
      local
	 left, top, width, height: INTEGER
      do
	 left := p1.x.min (p2.x)
	 top := p1.y.min (p2.y)
	 width := p1.x.max (p2.x) - left + 1
	 height := p1.y.max (p2.y) - top + 1
	 !! Result.make (left, top, width, height)
      end
   
   selection: RUBBER_BAND_SELECTION
	 -- the selection object
   
   step: INTEGER
	 -- if 0: we don't have a selection; if 1: we have a normal
	 -- rubber band selection (dragging phase); if 2: we're
	 -- modifying the position of individual points
   
   dragging: BOOLEAN
	 -- `True' if we are dragging something
   
   p1, p2: GDK_POINT
	 -- the corners used during `step=1'   
   
   expect_key_press: BOOLEAN
	 -- the last mouse click was with button 2 and we should 
	 -- expect the next character to try and change the order of 
	 -- the points in the selection
      
   x,y,jx,jy: INTEGER
   da: BOOLEAN
   random: STD_RAND
   image: GDK_IMAGE
   idle: GTK_IDLE
   gradient: COLLECTION[GDK_COLOR]

   line_drawn:BOOLEAN
   
   a1,b1: DOUBLE

   
end -- EFRACTAL_CANVAS

